#!/usr/bin/env tclsh

puts "
This program converts vmusic data base files created with the tcl/tk
version of the program to a new, ascii, version readable by the version 2.0
of VMusic. Once you've run Vmusic you can delete the ascii file.
	
Bob van der Poel <bvdpoel@kootenay.com>  Aug/2000
"

set dataFile ~/data/vmusic.dat
set outDir ~/data/vmusicXX
set outName $outDir/vmusic2.dat

set title "VMusic"

set musicdata {}

puts "Reading tcl file, $dataFile"
		
if [catch {open $dataFile r} infile] {
	puts "Can't open input file $dataFile"
	exit
}
	
# skip 2 comment lines...
	
gets $infile
gets $infile
	
set vs [gets $infile]
if { [lindex $vs 0] != $title } {
	puts "$title fatal error\n\
		The file '$dataFile' does not appear to be a valid $title database"
	exit
}
	
set vs [lindex $vs 1]
	
if { $vs != "1" && $vs != "2" } {
	puts "$title fatal error\n\
		I only know how to handle version 1 and 2 $title database files"
	exit
}
	
	
# read and discard geometries for window placements
	
set g [gets $infile]
	
# read and discard list of categories
	
set categories [gets $infile]
	
# read the database...
	
set musicdata {}
while { ![eof $infile] } {
	set r [gets $infile]
	if {$r == "" } continue
		
	# version 1 database file don't have a disk ID. Simple
	# enough to insert this while reading.
		
	if { $vs == "1" } { set r [linsert $r 1 {}] }
 	lappend musicdata $r
}
close $infile
	
	
# Save to new format

puts "Creating new version file, $outName"

file mkdir $outDir
set outfile [open $outName w]
	
puts $outfile "####  This file was created by the vmusic data file"
puts $outfile "####  conversion program so that your old vmusic-tcl"
puts $outfile "####  files can be read by the new vmusic-python progam"
puts $outfile "####  The first time you run the new program this file"
puts $outfile "####  will be converted to a binary format. After then"
puts $outfile "####  it's probably safe to delete this file."
puts $outfile ""
	

puts $outfile ""

set count 0	
foreach r $musicdata {	
	puts $outfile "TITLE:[lindex $r 0]"
	puts $outfile "ID:[lindex $r 1]"
	puts $outfile "LABEL:[lindex $r 2]"
	puts $outfile "CATEGORY:[lindex $r 3]"
	puts $outfile "COMMENT:[lindex $r 4]"
	puts $outfile "DATE: "
				
	set r [lrange $r 5 end]
	foreach {n a c t} $r {
		if { $n=="" } continue
		puts $outfile "NAME:$n"
		puts $outfile "ARTIST:$a"
		puts $outfile "COMPOSER:$c"
		puts $outfile "TIME:$t"
	}
	puts $outfile ""
	incr count
}
puts $outfile ""

close $outfile
	
puts "Conversion complete, $count records"

