/*	getcdtimes.c

	Scan an audio CD and report the length of each track. This was written
	to interface with vmusic so that the timings don't need to be typed for
	each song.
	
	I think this program is needed...I can't see a way to do ioctl()s from within
	tck/tk.
	
	The output is quite primitive, but okay for our purpose. If there are
	any command line opts (most likely -?) a helpful message is printed,
	otherwise the timing for this cd is printed to stdout.
	
	The code for this was lifted from the workman code (specially plat_linux.c).
	To adapt this to other platforms have a look at the platform specific code
	in workman...
	
	98/05/10  Bob van der Poel
*/


#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/param.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <sys/ioctl.h>
#include <linux/cdrom.h>
#include <getopt.h>

struct toc
{
	int min;
	int sec;
	int frame;
}cdtoc[100];

int trcount;

void do_track_lens(void);
void do_frame_info(void);
void read_frame_info(char *);
void usage(void);
unsigned long cddb_discid(int tot_trks);
int cddb_sum(int n);
char dototal=0;
char dosecs=0;


main(int argc, char *argv[])
{
	int t;
	char *device = "/dev/cdrom";
	char get_timings=1, get_id=1, print_frame=1;
	
	while((t=getopt(argc, argv, "d:itfcsvh?"))!=-1)
	{
		switch(t)
		{
			case 't':
				get_timings=0;
				break;
			case 'i':
				get_id=0;
				break;
			case 'f':
				print_frame=0;
				break;
				
			case 'v':
				fprintf(stderr, "getcdtimes version 1.2\n");
				exit(-1);
			
			case 'c':
				dototal=1;
				break;
				
			case 's':
				dosecs=1;
				break;
				
			default:
				usage();
		}
	}

	if (optind < argc) device = argv[optind];
	read_frame_info(device);
	
	if(get_id) printf("ID %08x\n", cddb_discid(trcount));
	if(print_frame) do_frame_info();
	if(get_timings) do_track_lens();
		
	exit(0);
}

// Calculate a cddb disk ID
// This is taken verbatim from the sample code on
// the cddb website

int
cddb_sum(int n)
{
	int	ret;

	/* For backward compatibility this algorithm must not change */

	ret = 0;

	while (n > 0) {
		ret = ret + (n % 10);
		n = n / 10;
	}

	return (ret);
}

unsigned long
cddb_discid(int tot_trks)
{
	int	i,
		t = 0,
		n = 0;

	/* For backward compatibility this algorithm must not change */

	i = 0;

	while (i < tot_trks) {
		n = n + cddb_sum((cdtoc[i].min * 60 ) + cdtoc[i].sec);
		i++;
	}

	t = ((cdtoc[tot_trks].min * 60 ) + cdtoc[tot_trks].sec) -
	    ((cdtoc[0].min * 60  ) + cdtoc[0].sec );


	return ((n % 0xff) << 24 | t << 8 | tot_trks);
}


// Display the frame offsets to stdout.


void
do_frame_info(void)
{
	int t;
	
	printf ("TF ");
	for(t=0; t<=trcount; t++)
	{
		printf("%d ", (cdtoc[t].min * 60 * 75) + 
			(cdtoc[t].sec * 75) +
			 cdtoc[t].frame); 
	}
	printf("\n");
}


// Display the track timings to stdout.

void
do_track_lens(void)
{
	int t, start, end, len, grandtotal;

	grandtotal=0;
	for(t=0; t<trcount; t++)
	{
		start = cdtoc[t].min*60 + cdtoc[t].sec;
		end =  cdtoc[t+1].min*60 + cdtoc[t+1].sec;
		len=end-start;
		grandtotal += len;
		
		if(dosecs) printf("%d %d\n", t+1, len);
		else printf("%d %d:%02d\n", t+1, len / 60, len % 60);
	}
	if(dototal)
		printf("Total: %d:%02d\n", grandtotal/60, grandtotal % 60);
}
	
// Read the frame info from the cd installed in drive

void
read_frame_info(char *device)
{
	int fd, track;
	struct cdrom_tochdr	hdr;
	struct cdrom_tocentry	entry;
	
	if((fd=open(device, O_NONBLOCK))<=0)
	{
		fprintf(stderr, "Can't access /dev/cdrom: %s\n", strerror(errno));
		exit (-1);
	}
	
	if (ioctl(fd, CDROMREADTOCHDR, &hdr))
	{
		fprintf(stderr, "Can't read CD header: %s\n", strerror(errno));
		exit (-1);
	}
	
	trcount = hdr.cdth_trk1;
	
	entry.cdte_track = CDROM_LEADOUT;
	entry.cdte_format = CDROM_MSF;
	
	if (ioctl(fd, CDROMREADTOCENTRY, &entry))
	{
		fprintf(stderr, "Can't read CD TOC entry: %s\n", strerror(errno));
		exit(-1);
	}
	
	cdtoc[trcount].min=entry.cdte_addr.msf.minute ;
	cdtoc[trcount].sec=entry.cdte_addr.msf.second ;
	cdtoc[trcount].frame=entry.cdte_addr.msf.frame;

	for(track=0; track<trcount; track++)
	{
		entry.cdte_track = track+1;
		entry.cdte_format = CDROM_MSF;
	
		if (ioctl(fd, CDROMREADTOCENTRY, &entry))
		{
			fprintf(stderr, "Can't read CD TOC entry: %S\n", strerror(errno));
			exit(-1);
		}
	
		cdtoc[track].min=entry.cdte_addr.msf.minute ;
		cdtoc[track].sec=entry.cdte_addr.msf.second ;
		cdtoc[track].frame=entry.cdte_addr.msf.frame;

	}
	close(fd);
}

// display usage message

void
usage(void)
{
	int t;
	
	char *msg[]=
	{
		"getcdtimes, (c) 1998/2001 Bob van der Poel",
		"Displays audio cd timings/disk id  to stdout.",
		"A part of the vmusic database program",
		"",
		"getcdtimes [opts] [device] [opts]",
		"Options:",
		"   -i   don't display disk ID",
		"   -t   don't display timings",
		"   -f   don't display frame info",
		"   -c   display grand total",
		"   -s   display seconds",
		"   -v   displays the version number",
		"   -h   this message",
		"",
		"The device option defaults to /dev/cdrom."
	};
	
		
	for(t=0; t<sizeof(msg)/sizeof(msg[0]); puts(msg[t++]));

	exit(-1);
}
